@@ -132,13 +132,11 @@ class Cell(object):
     def volume(self):
         """The volume of the cell."""
         a, b, c, alpha, beta, gamma = self.parameters
-        # Roundoff errors!
-        value = cos(alpha) * cos(beta) * cos(gamma)
-        if value < 0.0 and abs(value) < 1.0e-8:
-            value = 0.0
-        return a * b * c * (
-            1 - cos(alpha) ** 2 - cos(beta) ** 2 - cos(gamma) ** 2
-        ) + 2 * math.sqrt(value)
+        ca = cos(alpha)
+        cb = cos(beta)
+        cg = cos(gamma)
+        V = a * b * c * math.sqrt(1 - ca**2 - cb**2 - cg**2 + 2 * ca * cb * cg)
+        return V
 
     def equal(self, other, tol=1.0e-6):
         """Check if we are equal to another iterable to within a tolerance.
@@ -200,7 +198,7 @@ class Cell(object):
         sg = sin(gamma)
 
         twopi = 2 * math.pi
-        V = a * b * c * math.sqrt(1 - ca ** 2 - cb ** 2 - cg ** 2 + 2 * ca * cb * cg)
+        V = a * b * c * math.sqrt(1 - ca**2 - cb**2 - cg**2 + 2 * ca * cb * cg)
         # Transpose...
         # [1 / a, -cg / (a * sg), b * c * (ca * cg - cb) / (V * sg)],
         # [0, 1 / (b * sg), a * c * (cb * cg - ca) / (V * sg)],
@@ -272,7 +270,7 @@ class Cell(object):
         cg = cos(gamma)
         sg = sin(gamma)
 
-        V = a * b * c * math.sqrt(1 - ca ** 2 - cb ** 2 - cg ** 2 + 2 * ca * cb * cg)
+        V = a * b * c * math.sqrt(1 - ca**2 - cb**2 - cg**2 + 2 * ca * cb * cg)
         # Transpose of ...
         # [a, b * cg, c * cb],
         # [0, b * sg, c * (ca - cb * cg) / sg],
@@ -340,7 +338,7 @@ class Cell(object):
         cg = cos(gamma)
         sg = sin(gamma)
 
-        V = a * b * c * math.sqrt(1 - ca ** 2 - cb ** 2 - cg ** 2 + 2 * ca * cb * cg)
+        V = a * b * c * math.sqrt(1 - ca**2 - cb**2 - cg**2 + 2 * ca * cb * cg)
         # Transpose...
         # [1 / a, -cg / (a * sg), b * c * (ca * cg - cb) / (V * sg)],
         # [0, 1 / (b * sg), a * c * (cb * cg - ca) / (V * sg)],
@@ -375,6 +373,26 @@ class Cell(object):
         """
         return self.to_cartesians_transform(as_array=as_array)
 
+    def from_vectors(self, vectors):
+        """Set the cell parameters from the lattice vectors.
+
+        Parameters
+        ----------
+        vectors : [[float*3]*3]
+            The lattice vectors as a list [a, b, c]
+        """
+        va = vectors[0]
+        vb = vectors[1]
+        vc = vectors[2]
+        a = math.hypot(*va)
+        b = math.hypot(*vb)
+        c = math.hypot(*vc)
+        alpha = math.degrees(math.acos(dot(vb, vc) / (b * c)))
+        beta = math.degrees(math.acos(dot(va, vc) / (a * c)))
+        gamma = math.degrees(math.acos(dot(va, vb) / (a * b)))
+
+        self.parameters = [a, b, c, alpha, beta, gamma]
+
 
 class _Cell(Cell):
     """A class for handling cell parameters as part of MolSystem.
@@ -676,7 +694,7 @@ class _Cell(Cell):
         parameters = list(self._parameters)
         parameters.append(self.id)
         self.cursor.execute(
-            "UPDATE cell SET a=?, b=?, c=?, alpha=?, beta=?, gamma=?" "WHERE id = ?",
+            "UPDATE cell SET a=?, b=?, c=?, alpha=?, beta=?, gamma=? WHERE id = ?",
             parameters,
         )
         self.db.commit()
